#!/usr/bin/env python
# -*- coding: us-ascii -*-
#------------------------------------------------------
# Copyright (C) 2004-2005 ABINIT group (TD)
# This file is distributed under the terms of the
# GNU General Public License, see ~ABINIT/Infos/copyright
# or http://www.gnu.org/copyleft/gpl.txt .
# For the initials of contributors, see ~ABINIT/Infos/contributors .
#------------------------------------------------------
# Add a module for each routine in ABINIT
# in each Src_xxx which contains an interface
# for each subroutine inside a directory.
#Date: 14/07/2005
#------------------------------------------------------

#Plan
#----
# 1 - List all routines and store all in memory.
# 2 - Search all routines belonged to a directory.
# 3 - Remove all interfaces contained in the source files.
# 4 - Create a module containing the interface of all routines
#     for each directory.
# 5 - Add this file in the directory Src_defs.
#     (this is a choice which is not the best I think)
# 6 - Modify object_list of the directory Src_defs.
# 7 - Copy the new version

#TODO
#----
# Add correct children to _Src_xxx_ files.
# Add children and parents to the files interfaces_xxx.F90.
# Remove Tab (extension of Fortran95).
# Add an update mechanism.
# Check the hierarchy of the directories.
# Call after ; is not detected.
# Reduce as possible the intent(inout) specially if the argument is a dimension of arrays
# Problem with mes_abinit!!!

#Usage (and help)
def usage(error=0):
    print """
module [options] source
    where:
      source    is the directory that contains ABINIT sources
    options:
      --help    displays this message
      --single  Use one file of interfaces '%s.F90'
      --verbose displays more information
""" % single_interfaces
    sys.exit(error)

import os
import sys
import getopt
import re
import string

#Check the version of python
version = map(int,sys.version_info[0:3])
if  version < [2,3,0]:
    sys.stderr.write("Detected version %d.%d.%d\n" % tuple(version))
    sys.stderr.write("Minimal required version is python 2.3.0")
    sys.exit(1)

#Indentation
indent = " "
#Prefix for module containing interface.
prefix = "interfaces_"

#Regular expressions
#-------------------
re_amp = re.compile('^[ ]+&[ ]*', \
                       re.MULTILINE)
#To remove blank lines, comments and cpp directives
re_blank = re.compile('^(([ ]*?)|([ ]*[!#].*?))\n', \
                       re.MULTILINE)
#Detect call (not after an if or ;)
re_call = re.compile('(^[ ]+call[ ]*)(\w+)', \
                       re.MULTILINE+re.IGNORECASE)
re_comment_interface = re.compile('^!Interfaces.*?\n', \
                       re.MULTILINE)
#Use match and no search to have the first lines of comments.
re_comment_match = re.compile("^([ \t]*((!.*)|[ ]*)\n)+")
re_contains = re.compile('^[ ]*contains[ ]*\n', \
                       re.MULTILINE+re.IGNORECASE)

re_startblock = re.compile('(?P<header>[ \t]*(?P<type>(program|subroutine|(([^!\n]*?)function)))' \
                    + '[ ]*(?P<name>\w+)[ ]*([(][^)]*[)])?)[^\n]*\n', \
                       re.MULTILINE+re.IGNORECASE)
re_endblock   = re.compile('^[ \t]*end[ ]*(function|program|subroutine).*\n', \
                              re.MULTILINE+re.IGNORECASE)

re_include = re.compile('^[ ]*include.*?\n', \
                       re.MULTILINE+re.IGNORECASE)
re_implicit_none = re.compile('^[ ]*implicit[ ]*none', \
                       re.MULTILINE+re.IGNORECASE)
#Line containing implicit none + \n before
re_impnone_n = re.compile('[\n]*?[ ]*implicit[ ]*none.*?\n', \
                       re.MULTILINE+re.IGNORECASE)
re_indent = re.compile('^[ ]*(?!#)',re.MULTILINE)
re_interface = re.compile('^[ ]+interface.+?end[ ]+interface[ ]*\n', \
                       re.MULTILINE+re.IGNORECASE+re.DOTALL)
re_module = re.compile("^[ ]*(module|program)[ ]*(\w+)", \
                       re.MULTILINE+re.IGNORECASE)
re_use = re.compile('^[ ]+use[ ]*(?P<name>\w+)', \
                       re.MULTILINE+re.IGNORECASE)
re_use_prefix = re.compile('^[ ]+use[ ]*'+prefix+'.*?\n', \
                       re.MULTILINE+re.IGNORECASE)
#To remove not useful use for the interfaces (should be automatically determined)
re_use_unwanted = re.compile('^[ ]+use[ ]*(defs_interfaces|defs_xc|defs_xderive|flib_sax|f90_unix|netcdf|smoothvlocal)[ ]*?\n', \
                       re.MULTILINE+re.IGNORECASE)
re_use_replaced = re.compile('^[ ]+use[ ]*(defs_interfaces|defs_xc)[ ]*?\n', \
                       re.MULTILINE+re.IGNORECASE)

#Definition of the possible types
t_unknown = 0
t_file = 1
t_F90 = 2
t_module = 3
t_routine = 4
t_function = 5
t_comment = 6
t_header = 7
t_body = 8

def special_sort(x,y):
   "Special sort for the modules"
   if x[0] == "i":
       if y[0] == "i":
           return cmp(x,y)
       else:   
           return 1
   else:   
       if y[0] == "i":
           return -1
       else:
           return cmp(x,y)

#Class for ABINIT
class Project:
    "Class to define a project which is a set of directories and files."
    def __init__(self,dir,pattern_dir='.*',pattern_file='.*',name=""):
        "Initialisation"
        self.ROOT = dir
        self.dirs = []
        self.dirsfiles = dict()
        self.dirsfiles[dir] = []
        self.files = dict()
        self.routines = dict()
        self.name = name
        self.ndirs = 0
        self.nfiles = 0
        self.add(self.ROOT,pattern_dir,pattern_file)
        self.copied = 0
        mes_abinit.write("Project %s: Initialisation\n" % self.name,verbose=-10)
        mes_abinit.write("Number of directories: %d\n" % self.ndirs,verbose=-10)
        mes_abinit.write("Number of files: %d\n" % self.nfiles,verbose=-10)
    #
    def add(self,dir,pattern_dir='.*',pattern_file='.*'):
        "Add directories and files with a pattern"
        re_dir = re.compile(pattern_dir)
        re_file = re.compile(pattern_file)
        files = os.listdir(dir)
        for file in files:
            dd = "%s/%s" % (dir,file)
            if os.path.isdir(dd):
                if re_dir.match(file):
                    mes_abinit.write("[-----> %s: " % dd)
                    self.ndirs = self.ndirs + 1
                    self.dirs.append(dd)
                    self.dirsfiles[dd] = []
                    #Add all files in this sub-directory
                    self.add(dd,pattern_dir,pattern_file)
                    mes_abinit.write(" <-----]\n")
                else:
                    #We ignore it.
                    continue
            elif os.path.isfile(dd):
                if re_file.match(file):
                    self.add_file(dir,file)
    #
    def add_file(self,dir,file,create=False):
        "Add a file in the project."
        self.dirsfiles[dir].append(file)
        tt = file.split(".")
        mes_abinit.write("(%s)" % file)
        if len(tt) > 1 and tt[-1] == "F90":
            if "hdr_skip" in file or "hdr_io" in file:
                self.files[dir,file] = File_F90_HDR(file,dir=dir)
            else:
                self.files[dir,file] = File_F90(file,dir=dir)
        elif "_Lib_" in file:
            #Description file of a library
            self.files[dir,file] = File_F90(file,dir=dir)
        else:
            self.files[dir,file] = File(file,dir=dir)
        self.nfiles = self.nfiles + 1
        if not create:
            self.files[dir,file].read()
    #
    def add_use_interface(self):
        "Add 'use interface_' at each file."
        mes_abinit.write("Add 'use interface_...' in each routine\n",verbose=-10)
        #Do for each routine stored in self.routines
        for name in self.routines.keys():
            mes_abinit.write("(%s)" % name)
            self.routines[name].add_use_interface(self.routines,self.re_functions)
    #
    def analyse_all(self):
        "Analyse all the project."
        mes_abinit.write("Analyse all the project\n",verbose=-10)
        mes_abinit.flush()
        for (dir,file) in self.files:
            self.files[dir,file].analyse(self.routines)
        #Determine the list of functions
        self.set_functions()
    #
    def copy_all(self,NEW):
        "Copy all the project in a newdir."
        mes_abinit.write("Copy all the project in %s\n" % NEW,verbose=-10)
        mes_abinit.flush()
        for (dir,file) in self.files:
            newdir = dir.replace(self.ROOT+"/",NEW+"/")
            if not os.path.exists(newdir):
                os.mkdir(newdir)
            self.files[dir,file].write(newdir,file)
            self.copied = self.copied + self.files[dir,file].copied
        if self.copied <= 1:
            mes_abinit.write("%d copied file.\n" % self.copied)
        else:
            mes_abinit.write("%d copied files.\n" % self.copied)
        #Statistics
        mes_abinit.final()
    #
    def interfaces_all(self):
        "Build the interfaces of all files"
        for dir in self.dirs:
            dir_module = "%s/Src_defs" % self.ROOT 
            main = os.path.basename(dir)
            if main == "Src_main" or main == "Src_defs":
                continue
            if single:
                module = single_interfaces
            elif "Src_" in main:
                module = "%s%s" % (prefix,main.replace("Src_",""))
            elif "Lib_" in main:
                module = "%s%s" % (prefix,main)
            else:
                mes_abinit.fatal("Interface for this directory %s!!\n" % main)
            file_module = "%s.F90" % module
            self.add_file(dir_module,file_module,create=True)
            pstruct = self.files[dir_module,file_module]
            pstruct.code = head_source % { 'name': module, 'dir': dir[2:] }
            #Add interfaces in the file
            list = self.dirsfiles[dir]
            for file in list:
                pstruct.code = pstruct.code + "\n" + self.files[dir,file].interface()
            pstruct.code = pstruct.code \
                           + "\n" \
                           + "end module %s\n" % module \
                           + "!!***\n"
    #
    def object_list(self,dir):
        "Create the file 'object_list'"
        file = "object_list"
        self.add_file(dir,file,create=True)
        pstruct = self.files[dir,file]
        list = self.dirsfiles[dir]
        list.sort(special_sort)
        pstruct.code = "OBJS.o ="
        for obj in list:
            if obj[-3:] == "F90":
                pstruct.code = pstruct.code + "\\"+"\n" + "\t%s" % obj
        pstruct.code = pstruct.code.replace(".F90",".o")+"\n"
    #
    def set_called_routines(self):
        "Build the list of called_routines"
        mes_abinit.write("Determine the called routines of all routines\n",verbose=-10)
        mes_abinit.flush()
        for struct in self.routines.values():
            #if struct.type == t_function:
            struct.set_called(self.re_functions)
    #
    def set_functions(self):
        "Build the list of functions"
        self.functions = []
        functions = ""
        mes_abinit.write("\n")
        for struct in self.routines.values():
            if struct.type == t_function:
                mes_abinit.write("Functions --> %s\n" % struct.name)
                self.functions.append(struct.name)
                functions = "%s|%s" % (functions,struct.name)
        nf = len(self.functions)
        mes_abinit.write("Number of functions: %d\n" % nf,verbose=-10)
        if nf  == 0:
            self.re_functions = None
        else:
            self.re_functions = re.compile("(?<![a-z])(%s)[ ]*[(]" % functions[1:],re.IGNORECASE)
    #
    def special(self,special_modif):
        "Do some special modifications"
        mes_abinit.write("\nSpecial transformations:\n")
        for name in special_modif:
            mes_abinit.write("[special:%s]" % name)
            eval("self.routines[name].%s" % special_modif[name])
        mes_abinit.write("\n")
    #
    def statistics(self):
        "Display some statistics about the project"
        mes_abinit.write("Project %s:\n" % self.name,verbose=-10)
        mes_abinit.write("Number of directories: %d\n" % self.ndirs,verbose=-10)
        mes_abinit.write("Number of files: %d\n" % self.nfiles,verbose=-10)
        mes_abinit.write("Number of routines: %d\n" % len(self.routines),verbose=-10)


#General class handling File and the other ones.
class Structure:
    "Define a structure (a routine, header, etc) which can have a portion of code"
    def __init__(self,name,type,parent=None):
        self.name = name
        self.type = type
        self.children = []
        self.parent = parent
        self.code = ""
    #Read
    def read(self,fd):
        "Read from a file descriptor"
        self.code = fd.read()
    #Write
    def write(self,fd):
        "write into a file descriptor"
        fd.write(self.code)
        for child in self.children:
           child.write(fd)
    #Analyse
    def analyse(self):
        "Do nothing"
        pass
    #Interface
    def interface(self,routines=None):
        "Do nothing"
        return ""


#Class for all files
class File(Structure):
    "Class to define a file which is a code with a structure"
    def __init__(self,name,dir=".",type=t_file):
        "Initialisation"
        Structure.__init__(self,name,type,parent=None)
        self.dir = dir
        self.children = []
        self.copied = 0
        #Time of last modification
        self.mtime = 0
        self.type = type
    #
    def read(self,fd=None):
        "Read the file."
        if fd == None:
            fd = open("%s/%s" % (self.dir,self.name))
        self.mtime = os.fstat(fd.fileno())[-1]
        self.code = fd.read()
        fd.close()
    #
    def write(self,dir,file,fd=None):
        "Write the file as '%s/%s' % (dir,file)."
        mes_abinit.write("[%s/%s]" % (dir,file),verbose=10)
        if fd == None:
            fd = open("%s/%s" % (dir,file),"w")
        fd.write(self.code)
        for child in self.children:
            child.write(fd)
        fd.close()
        self.copied = self.copied + 1


#Class for Fortran90 files
class File_F90(File):
    "Class to define a fortran file which is a code with a structure"
    def __init__(self,name,dir="."):
        "Initialisation"
        File.__init__(self,name,dir,type=t_F90)
        if single:
            self.module = single_interfaces
        else:
            self.module = os.path.basename(self.dir)
            self.module = "%s%s" % (prefix,self.module.replace("Src_",""))
    #
    def analyse(self,routines=None):
        "Analyse the code of the file"
        mes_abinit.write("[%s/%s:" % (self.dir,self.name),verbose=10)
        #If there is the statement "contains", we skip the routine
        if  re_contains.search(self.code):
            mes_abinit.error("[%s/%s]" % (self.dir,self.name) \
                             + " 'contains' statement detected\n" \
                             + "--> We ignore this file!" )
            return
        #Is it a module or a program?
        if  re_module.search(self.code):
            #We assume only one module per file and no subroutines inside
            pstruct = Structure("module",t_module,parent=self)
            pstruct.code = self.code
            self.code = ""
            self.children.append(pstruct)
            return
        if re_interface.search(self.code):
            #No interface are permitted in a file.
            mes_abinit.warning("[%s/%s] Interface detected\n   --> We remove it!" \
                               % (self.dir,self.name))
            #Remove it.
            self.code = re_interface.sub('',self.code)
            #Remove also the comment !Interface
            self.code = re_comment_interface.sub('',self.code)
        while self.code != "":
            has = False
            comment = re_comment_match.match(self.code)
            if comment:
                pstruct = Structure("comment",t_comment,parent=self)
                pstruct.code = self.code[comment.start():comment.end()]
                self.code = self.code[comment.end():]
                self.children.append(pstruct)
                has = True
            routine = re_startblock.match(self.code)
            if routine:
                name = routine.groupdict()['name']
                if name == None:
                    print "[",routine.group(),"]"
                    print routine.groupdict()
                    sys.exit(0)
                mes_abinit.write("<%s>" % name)
                lstart = routine.start()
                if lstart != 0:
                    mes_abinit.fatal("Missing characters for %s/%s:%s\n" % \
                                     (self.dir,self.name,name))
                endblock = re_endblock.search(self.code)
                if endblock:
                    lend = endblock.end()
                else:
                    mes_abinit.fatal("No end block for %s/%s:%s\n" % (self.dir,self.name,name))
                pstruct = Routine(name,t_routine,routine,self.code[lstart:lend],parent=self)
                #Add in the dictionary of all routines
                if routines != None:
                    routines[name] = pstruct
                #Analyse the routine
                pstruct.analyse()
                self.code = self.code[lend:]
                self.children.append(pstruct)
                has = True
            if not has:
                mes_abinit.write("\n"+self.code)
                mes_abinit.fatal("Analysis Error in %s/%s\n" % (self.dir,self.name))
        mes_abinit.write("]",verbose=10)
        mes_abinit.flush()
    #
    def interface(self):
        "Build the interface"
        code = ""
        for child in self.children:
            if child.type == t_routine or child.type == t_function:
                code = code + child.interface()
        return code
    #
    def special(self,modif):
        "Special modifications"
        for child in self.children:
            if child.type == t_routine:
                eval("child.%s" % modif)


#Special class for hdr_skip, hdr_io, hdr_io_netcdf.F90 Fortran90 files
class File_F90_HDR(File_F90):
    "Class to define a fortran file which is a code with a structure"
    def __init__(self,name,dir="."):
        "Initialisation"
        File_F90.__init__(self,name,dir)
    #
    def analyse(self,routines=None):
        "Analyse the code of the file"
        File_F90.analyse(self,routines)
        #Add a routine which has the name of the file
        name = self.name.replace(".F90","")
        pstruct = Routine(name,t_routine,parent=self)
        routines[name] = pstruct
    #
    def interface(self):
        "Build the interface"
        code = ""
        name = self.name.replace(".F90","")
        first = True
        for child in self.children:
            if child.type == t_routine:
                code_child = child.interface()
                code_child = code_child.replace(indent+"end interface","")
                if first:
                    code_child = code_child.replace("interface\n","interface %s\n\n" % name)
                    first = False
                else:
                    code_child = code_child.replace(indent+"interface","")
                code = code + code_child
        code = code + indent + "end interface %s\n" % name
        return code


#Class for routines
class Routine(Structure):
    "Class to handle subroutines or function"
    def __init__(self,name,type,search=None,code="",parent=None):
        Structure.__init__(self,name,type,parent)
        self.called = []
        if search != None:
            self.header = search.groupdict()['header']
            self.nature = search.groupdict()['type']
            if "function" in self.nature:
                self.type = t_function
                self.nature = "function"
        else:
            self.header = None
            self.nature = None
        self.code = code
        self.file = self.parent.name
        self.dir = self.parent.dir
        self.module = self.parent.module
        #No interface defined for this routine (call self.interface to do that)
        self.has_interface = False
    #
    def add_use_interface(self,routines,re_functions):
        "Add 'use self.module'"
        if self.header == None:
            mes_abinit.warning("[%s/%s:%s] No header for this routine" \
                                   % (self.dir,self.file,self.name))
            return
        modules = []
        #Add routines
        for sub in self.called:
            try:
                psub = routines[sub]
                module = psub.module
                if not modules.count(module):
                    modules.append(module)
                if psub.type == t_function:
                    #We remove the declaration of the function
                    self.code = re.sub( \
                                "(.*::.*),%s(?=[, \n])" % psub.name, r"\1",self.code)
                    self.code = re.sub( \
                                "(.*::.*)%s(?=[, \n])" % psub.name, r"\1",self.code)
                    #We remove a comma behind "::"
                    self.code = re.sub( "::[ ]*,", ":: ",self.code)
                    #We remove a line finishing by "::" with a comma or not behind
                    self.code = re.sub( ".*::[ ]*[,]*[ ]*\n", "",self.code)
            except KeyError:
                mes_abinit.warning("[%s/%s:%s] No interface for this routine" \
                                   % (self.dir,self.file,sub))
        modules.sort()
        text_use="\n"
        for module in modules:
            if prefix in module:
                if module == self.module and self.has_interface:
                    #We except the given subroutine
                    text_use = text_use + indent + "use %s, except_this_one => %s\n" \
                               % (self.module,self.name)
                else:
                    text_use = text_use + indent + "use %s\n" % module
        text_use = text_use + "\n" + indent + "implicit none\n"
        header = self.header
        string = re.escape(header)
        re_header = re.compile(string +".*?^[ ]*implicit[ ]*none[ ]*\n",re.MULTILINE+re.DOTALL)
        try:
            new_header = re_header.findall(self.code)[0]
        except IndexError:
            mes_abinit.warning("[%s/%s:%s] No correct header for this routine" \
                                   % (self.dir,self.file,self.name))
            return
            #print self.dir,self.file,self.name
            #print header
            #print "&"*40
            #print "Error in header: please modify the header of %s in %s" %(self.name,self.file)
            #print "&"*40
            #print self.code
            #sys.exit(0)
        #Remove line with use prefix
        new_header = re_use_prefix.sub('',new_header)
        #Remove some replaced use
        new_header = re_use_replaced.sub('',new_header)
        #Add text_use before implicit none
        new_header = re_impnone_n.sub(text_use,new_header)
        #Replace the header by the new header
        self.code = re_header.sub(new_header,self.code)
    #
    def interface(self):
        "Build the interface"
        if self.header == None:
            mes_abinit.warning("[%s/%s:%s] No header for this routine" \
                                   % (self.dir,self.file,self.name))
            return ""
        string = re.escape(self.header)
        
        re_special = re.compile(string +".*?"+"(?=!Local)",re.MULTILINE+re.DOTALL)
        code = re_special.findall(self.code)
        if len(code) == 0:
            #The format is not respected!
            mes_abinit.error( \
                    "[%s/%s:%s] Bad formatting for Arguments --> We ignore this routine." \
                    % (self.dir,self.file,self.name))
            self.has_interface = False
            return ""
        #Replace tabulations by space character
        code = code[0].replace("\t"," ")
        #Check the presence of implicit none
        if not re_implicit_none.search(code):
            #Fatal Error (problem with the script)
            mes_abinit.fatal("[%s] Implicit None is required!" % self.name)
        #Look for "use module".
        all_use = re_use.findall(code)
        #Remove use added by this program.
        code = re_use_prefix.sub('',code)
        #Remove unwanted use
        code = re_use_unwanted.sub('',code)
        #Remove include
        code = re_include.sub('',code)
        #Remove comments, blank lines and preprocessing directives.
        code = re_blank.sub('',code)
        #Indentation
        code = re_indent.sub(indent*3,code[:-1])
        #Add the end and unindent the first line
        code = code[1:] + "\n" + indent*2 + "end %s %s\n" % (self.nature,self.name)
        #Desindent a line beginning by &
        code = re_amp.sub('&'+indent*2,code)
        code =  indent + "interface\n" \
               + code \
               + indent + "end interface\n"
        self.has_interface = True
        return code
    #
    def replace(self,old,new):
        "Replace the regular expressions 'old' by the string 'new'"
        self.code = re.sub(old,new,self.code)
    #
    def set_called(self,re_functions):
        "Create the list of called routines"
        list = re_call.findall(self.code)
        mes_abinit.write("(%s:" % self.name)
        for tt in list:
            self.called.append(tt[1])
        #Add functions
        if re_functions != None:
            list = re_functions.findall(self.code)
            for tt in list:
                if tt != self.name:
                    if not self.called.count(tt):
                        self.called.append(tt)
        if len(self.called) > 0:
            for tt in self.called:
                mes_abinit.write("{%s}" % tt)
        mes_abinit.write(")")

class Message:
    "Message management"
    def __init__(self,verbose=0):
        "Initialisation"
        self.terrors = []
        self.twarnings = []
        self.verbose = verbose
    def error(self,text):
        "Display an error message"
        tt = "Error[%d]: %s\n" % (len(self.terrors)+1,text)
        self.terrors.append(tt)
        sys.stderr.write(tt)
    def warning(self,text,verbose=0):
        "Display a warning message"
        tt = "Warning[%d]: %s\n" % (len(self.twarnings)+1,text)
        self.twarnings.append(tt)
        if verbose <= self.verbose:
            sys.stderr.write(tt)
    def write(self,text,verbose=0):
        "Display a message"
        if verbose <= self.verbose:
           sys.stdout.write(text)
    def flush(self):
        "Flush the buffer"
        sys.stdout.flush()
    def fatal(self,text):
        "Display an error message and stops"
        tt = "Fatal Error: %s\n" % text
        sys.stderr.write(tt)
        sys.exit(1)
    def final(self):
        "Final statistics: Write in a log file and resume in stderr."
        fd = open("abilint.log","w")
        for tt in self.twarnings:
            fd.write(tt)
        for tt in self.terrors:
            fd.write(tt)
        fd.write("%d warnings.\n" % (len(self.twarnings)))
        fd.write("%d errors.\n" % (len(self.terrors)))
        sys.stderr.write("%d warnings.\n" % (len(self.twarnings)))
        sys.stderr.write("%d errors.\n" % (len(self.terrors)))
        sys.stderr.write("See the file 'abilint.log'\n")

#Header of the interface module.
head_source = \
"""!!****m* ABINIT/%(name)s
!! NAME
!! %(name)s
!!
!! FUNCTION
!! This module contains the interface of the routines
!! in the directory %(dir)s
!!
!! COPYRIGHT
!! Copyright (C) 2005 ABINIT group
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! NOTES
!! THIS FILE IS GENERATED AUTOMATICALLY BY abilint.py.
!!
!! SOURCE

module %(name)s
"""

special_modif = \
              { 
                "angle_dihedral": r"replace('.*::[ ]*bond_length[ ]*\n','')",
                "dotprod_g":      r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "dotprodm_v":     r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "dotprodm_vn":    r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "dotprod_v":      r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "dotprod_vn":     r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "matrixelmt_g":   r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "mean_fftr":      r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "meanvalue_g":    r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "sqnorm_g":       r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "sqnormm_v":      r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "sqnorm_v":       r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "clnmpi_fft":     r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')",
                "clnmpi_gs":      r"replace('\n[ ]*\n[ ]*type\(MPI_type\)','\n type(MPI_type)')"
                }


#Start the script
if __name__ == "__main__":
    #Check arguments
    single_interfaces = "interfaces_all"
    try:
        optlist, args = getopt.getopt(sys.argv[1:],"hsv",["help","single","verbose"])
    except getopt.error:
        print "Error in arguments"
        usage(error=1)
    #Help message
    single = False
    verbose = -10
    for opt,a in optlist:
        if opt == "-h" or opt == "--help":
            usage(error=0)
        elif opt == "-s" or opt == "--single":
            single = True
        elif opt == "-v" or opt == "--verbose":
            verbose = 10
    #Two arguments are required
    if len(args) != 1:
        sys.stderr.write('You have to specify the ABINIT source directory.\n')
        usage(error=1)
    #Define SRC and NEW directories
    SRC = args[0]
    NEW = SRC
    #Instantiation of the class message
    mes_abinit = Message(verbose=verbose)
    #Create the project and read all files
    abinit = Project(SRC,pattern_dir="Src_",pattern_file=".*[.]F90$",name="ABINIT")
    #Add the description file of each library
    abinit.add(SRC,pattern_dir="Lib_",pattern_file="_.*_")
    #Analyse the project.
    abinit.analyse_all()
    #Set the called routines
    abinit.set_called_routines()
    #Special modifications (to be done once).
    abinit.special(special_modif)
    #Create a file for all the interface and put it in "Src_defs".
    abinit.interfaces_all()
    #Add "use interface_" and remove the declarations of functions
    abinit.add_use_interface()
    #Modify the file object_list of the directory Src_defs.
    abinit.object_list("%s/Src_defs" % SRC)
    #We copy everything
    abinit.copy_all(NEW)
    abinit.statistics()
